using System;
using System.Windows.Forms;
using System.Reflection;
using System.ComponentModel;


namespace Tabdee.FormController
{
    public class FormController
    {
        private static Form mainEmbeddedForm = null;

        private static void ClearEmbeddedForms(Control containerControl)
        {
            foreach (Control control in containerControl.Controls) 
            {
                if (control.GetType().Equals(MainEmbeddedFormType)) 
                {
                    // Merely hide the main embedded form so we can restore it in the same state if required
                    control.Hide();
                    control.Enabled = false;
                }
                else if (control is Form) 
                {
                    // Close all other forms
                    (control as Form).Close();
                    control.Dispose();
                }
            }
        }

        private static bool ContainsForm(Control containerControl)
        {
            foreach (Control control in containerControl.Controls) 
            {
                if (control is Form) 
                {
                    return true;
                }
            }

            return false;
        } 

        private static Form ConstructNewForm(Type formType,params object[] constructorParameters)
        {
            Type[] parameterTypes = new Type[constructorParameters.Length];

            for (int i = 0; i < constructorParameters.Length; i++)
            {
                parameterTypes[i] = constructorParameters[i].GetType();
            }

            ConstructorInfo constructorInfo = formType.GetConstructor(parameterTypes);

            if (constructorInfo == null) 
            {
                throw new Exception("Unable to find constructor for " + formType.Name);
            }

            return (Form)constructorInfo.Invoke(constructorParameters);
        }

        private static void SetContainerFormProperties(Control containerControl,Form formToEmbed)
        {
            Form containerForm = (containerControl.TopLevelControl as Form);

            if (containerForm != null) 
            {
                // Show menu if there is one (null hides the menu)
                // Toolbars are difficult/impossible to hide
                containerForm.Menu = formToEmbed.Menu;

                // Show the OK button only if this is not the main embedded form
                // MinimizeBox = true displays the minimise button (X)
                // MinimizeBox = false displays the close button (OK)
                containerForm.MinimizeBox = formToEmbed.GetType().Equals(MainEmbeddedFormType);
            }
        }

        private static void EmbedFormInControl(Type formType,Control containerControl,params object[] constructorParameters)
        {
            ClearEmbeddedForms(containerControl);

            Form formToEmbed = ConstructNewForm(formType,constructorParameters);

            // Setting these properties to true stops the Start menu flicker
            formToEmbed.ControlBox = true;
            formToEmbed.MinimizeBox = true;
            formToEmbed.WindowState = FormWindowState.Maximized;
            formToEmbed.Parent = containerControl;
            SetContainerFormProperties(containerControl,formToEmbed);
            formToEmbed.Show();        

            if (formType.Equals(MainEmbeddedFormType))
            {
                // If this is the form for the main view then keep track of it
                mainEmbeddedForm = formToEmbed;
            }            
        }

        public static Type MainEmbeddedFormType = null;
        public static Form MainForm = null;

        public static void ShowMainEmbeddedForm()
        {
            if (mainEmbeddedForm == null)            
            {
                // Always embeds the new form in the main window
                EmbedFormInControl(MainEmbeddedFormType,MainForm);
            }
            else 
            {
                ClearEmbeddedForms(MainForm);
                mainEmbeddedForm.Enabled = true;
                SetContainerFormProperties(MainForm,mainEmbeddedForm);
                mainEmbeddedForm.Show();
            }
        }

        public static void ShowSubForm(Type formType,params object[] constructorParameters)
        {
            // Always embeds the new form in the main window
            EmbedFormInControl(formType,MainForm,constructorParameters);
        }

        public static bool ShowingForm
        {
            get { return ContainsForm(MainEmbeddedForm); }
        }

        public static Form MainEmbeddedForm
        {
            get { return mainEmbeddedForm; }
        }

        public static void Finalise()
        {
            ClearEmbeddedForms(MainForm);

            if (mainEmbeddedForm != null)
            {
                mainEmbeddedForm.Close();
                mainEmbeddedForm.Dispose();
            }
        }
    }
}
